#include <xsTypes.h>
#include <functionMap.h>
#include <stlToCArrays.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

void xszgau(const RealArray& energyArray, const RealArray& params,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{
   // ---
   //  XSPEC model subroutine
   //  Simple gaussian line profile
   // ---
   //  see ADDMOD for parameter descriptions
   //  number of model parameters:3
   //        1       EL      line energy (in energy units, e.g. keV)
   //        2       W	line wiflh (sigma) (in energy units)
   // 	3 	redshift
   //  intrinsic energy range: none
   //        N.B. the line may have significant flux at unphysical (i.e. negative)
   //             energies when EL<~W.
   //  algorithm:
   //        n(E) = (1./(sqrt(2pi*W*W)) * exp(-0.5*((E-EL)/W)**2)) dE
   //        N.B. the norm is equal to the total integrated number of counts in
   //             the line.
   //        If W is less than or equal to zero, the line is treated as a delta
   //             function.
   //        N.B. when the energy spacing is much greater than W and the stepsize
   //             for EL then the partial derivative determinations can lead
   //             to fit error conditions.  The solution is to increase the
   //             stepsize for EL
   // ---
   //  Just calls gaussian line model with redshifted energies.
   
   // Translated from xszgau.f Mar 09 (CG)
   
   using namespace XSutility;
   
   // fill energy array in source frame.

   int ne = energyArray.size() - 1;
   int ne1 = ne + 1;
   auto_array_ptr<float> apEz(new float[ne1]); 
   float *ez = apEz.get();
   double zfac = 1.0 + params[2];
   for (int ie=0; ie<ne1; ++ie)
      ez[ie] = static_cast<float>(energyArray[ie]*zfac);
      
   // Calculate gaussian
   
   float *ear=0, *pars=0, *photar=0, *photer=0;
   XSFunctions::stlToFloatArrays<float>(energyArray, params, flux, fluxErr,
           ear, pars, photar, photer);
   auto_array_ptr<float> apEar(ear);
   auto_array_ptr<float> apPars(pars);
   auto_array_ptr<float> apPhotar(photar);
   auto_array_ptr<float> apPhoter(photer);
   
   xsgaul_(ez, ne, pars, spectrumNumber, photar, photer);
     
   XSFunctions::floatFluxToStl<float>(photar, photer, ne, false, flux, fluxErr);
   
   // Include time dilation factor
   
   flux /= zfac;       
}
